@@ -4,22 +4,20 @@
 
     Image converter extension for Sphinx
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+
 import subprocess
 from subprocess import CalledProcessError, PIPE
+from typing import Any, Dict
 
+from sphinx.application import Sphinx
 from sphinx.errors import ExtensionError
 from sphinx.locale import __
 from sphinx.transforms.post_transforms.images import ImageConverter
 from sphinx.util import logging
 
-if False:
-    # For type annotation
-    from typing import Any, Dict  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
 
 logger = logging.getLogger(__name__)
 
@@ -29,10 +27,10 @@ class ImagemagickConverter(ImageConverter):
         ('image/svg+xml', 'image/png'),
         ('image/gif', 'image/png'),
         ('application/pdf', 'image/png'),
+        ('application/illustrator', 'image/png'),
     ]
 
-    def is_available(self):
-        # type: () -> bool
+    def is_available(self) -> bool:
         """Confirms the converter is available or not."""
         try:
             args = [self.config.image_converter, '-version']
@@ -40,7 +38,7 @@ class ImagemagickConverter(ImageConverter):
             subprocess.run(args, stdout=PIPE, stderr=PIPE, check=True)
             return True
         except OSError:
-            logger.warning(__('convert command %r cannot be run.'
+            logger.warning(__('convert command %r cannot be run, '
                               'check the image_converter setting'),
                            self.config.image_converter)
             return False
@@ -50,8 +48,7 @@ class ImagemagickConverter(ImageConverter):
                            exc.stderr, exc.stdout)
             return False
 
-    def convert(self, _from, _to):
-        # type: (str, str) -> bool
+    def convert(self, _from: str, _to: str) -> bool:
         """Converts the image to expected one."""
         try:
             # append an index 0 to source filename to pick up the first frame
@@ -65,7 +62,7 @@ class ImagemagickConverter(ImageConverter):
             subprocess.run(args, stdout=PIPE, stderr=PIPE, check=True)
             return True
         except OSError:
-            logger.warning(__('convert command %r cannot be run.'
+            logger.warning(__('convert command %r cannot be run, '
                               'check the image_converter setting'),
                            self.config.image_converter)
             return False
@@ -75,8 +72,7 @@ class ImagemagickConverter(ImageConverter):
                                  (exc.stderr, exc.stdout))
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_post_transform(ImagemagickConverter)
     app.add_config_value('image_converter', 'convert', 'env')
     app.add_config_value('image_converter_args', [], 'env')
