@@ -4,7 +4,7 @@
 
     Builder superclass for all builders.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import gettext
@@ -12,27 +12,27 @@ import os
 import re
 import warnings
 from collections import namedtuple
-from datetime import datetime
+from datetime import datetime, timezone
 from os import path
+from typing import Callable, Generator, List, Set, Tuple
 
 import babel.dates
 from babel.messages.mofile import write_mo
 from babel.messages.pofile import read_po
 
-from sphinx.deprecation import RemovedInSphinx30Warning, RemovedInSphinx40Warning
+from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.errors import SphinxError
 from sphinx.locale import __
 from sphinx.util import logging
 from sphinx.util.matching import Matcher
 from sphinx.util.osutil import SEP, canon_path, relpath
 
-
-logger = logging.getLogger(__name__)
-
 if False:
     # For type annotation
-    from typing import Callable, Generator, List, Set, Tuple  # NOQA
-    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.environment import BuildEnvironment
+
+
+logger = logging.getLogger(__name__)
 
 LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
 
@@ -40,33 +40,27 @@ LocaleFileInfoBase = namedtuple('CatalogInfo', 'base_dir,domain,charset')
 class CatalogInfo(LocaleFileInfoBase):
 
     @property
-    def po_file(self):
-        # type: () -> str
+    def po_file(self) -> str:
         return self.domain + '.po'
 
     @property
-    def mo_file(self):
-        # type: () -> str
+    def mo_file(self) -> str:
         return self.domain + '.mo'
 
     @property
-    def po_path(self):
-        # type: () -> str
+    def po_path(self) -> str:
         return path.join(self.base_dir, self.po_file)
 
     @property
-    def mo_path(self):
-        # type: () -> str
+    def mo_path(self) -> str:
         return path.join(self.base_dir, self.mo_file)
 
-    def is_outdated(self):
-        # type: () -> bool
+    def is_outdated(self) -> bool:
         return (
             not path.exists(self.mo_path) or
             path.getmtime(self.mo_path) < path.getmtime(self.po_path))
 
-    def write_mo(self, locale):
-        # type: (str) -> None
+    def write_mo(self, locale: str) -> None:
         with open(self.po_path, encoding=self.charset) as file_po:
             try:
                 po = read_po(file_po, locale)
@@ -84,16 +78,15 @@ class CatalogInfo(LocaleFileInfoBase):
 class CatalogRepository:
     """A repository for message catalogs."""
 
-    def __init__(self, basedir, locale_dirs, language, encoding):
-        # type: (str, List[str], str, str) -> None
+    def __init__(self, basedir: str, locale_dirs: List[str],
+                 language: str, encoding: str) -> None:
         self.basedir = basedir
         self._locale_dirs = locale_dirs
         self.language = language
         self.encoding = encoding
 
     @property
-    def locale_dirs(self):
-        # type: () -> Generator[str, None, None]
+    def locale_dirs(self) -> Generator[str, None, None]:
         if not self.language:
             return
 
@@ -103,8 +96,7 @@ class CatalogRepository:
                 yield locale_dir
 
     @property
-    def pofiles(self):
-        # type: () -> Generator[Tuple[str, str], None, None]
+    def pofiles(self) -> Generator[Tuple[str, str], None, None]:
         for locale_dir in self.locale_dirs:
             basedir = path.join(locale_dir, self.language, 'LC_MESSAGES')
             for root, dirnames, filenames in os.walk(basedir):
@@ -119,15 +111,13 @@ class CatalogRepository:
                         yield basedir, relpath(fullpath, basedir)
 
     @property
-    def catalogs(self):
-        # type: () -> Generator[CatalogInfo, None, None]
+    def catalogs(self) -> Generator[CatalogInfo, None, None]:
         for basedir, filename in self.pofiles:
             domain = canon_path(path.splitext(filename)[0])
             yield CatalogInfo(basedir, domain, self.encoding)
 
 
-def find_catalog(docname, compaction):
-    # type: (str, bool) -> str
+def find_catalog(docname: str, compaction: bool) -> str:
     warnings.warn('find_catalog() is deprecated.',
                   RemovedInSphinx40Warning, stacklevel=2)
     if compaction:
@@ -138,8 +128,7 @@ def find_catalog(docname, compaction):
     return ret
 
 
-def docname_to_domain(docname, compation):
-    # type: (str, bool) -> str
+def docname_to_domain(docname: str, compation: bool) -> str:
     """Convert docname to domain for catalogs."""
     if compation:
         return docname.split(SEP, 1)[0]
@@ -147,8 +136,8 @@ def docname_to_domain(docname, compation):
         return docname
 
 
-def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
-    # type: (str, str, List[str], str, bool) -> List[str]
+def find_catalog_files(docname: str, srcdir: str, locale_dirs: List[str],
+                       lang: str, compaction: bool) -> List[str]:
     warnings.warn('find_catalog_files() is deprecated.',
                   RemovedInSphinx40Warning, stacklevel=2)
     if not(lang and locale_dirs):
@@ -161,10 +150,9 @@ def find_catalog_files(docname, srcdir, locale_dirs, lang, compaction):
     return files
 
 
-def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact=None,
-                              charset='utf-8', force_all=False,
-                              excluded=Matcher([])):
-    # type: (List[str], str, List[str], bool, str, bool, Matcher) -> Set[CatalogInfo]
+def find_catalog_source_files(locale_dirs: List[str], locale: str, domains: List[str] = None,
+                              charset: str = 'utf-8', force_all: bool = False,
+                              excluded: Matcher = Matcher([])) -> Set[CatalogInfo]:
     """
     :param list locale_dirs:
        list of path as `['locale_dir1', 'locale_dir2', ...]` to find
@@ -180,9 +168,6 @@ def find_catalog_source_files(locale_dirs, locale, domains=None, gettext_compact
     """
     warnings.warn('find_catalog_source_files() is deprecated.',
                   RemovedInSphinx40Warning, stacklevel=2)
-    if gettext_compact is not None:
-        warnings.warn('gettext_compact argument for find_catalog_source_files() '
-                      'is deprecated.', RemovedInSphinx30Warning, stacklevel=2)
 
     catalogs = set()  # type: Set[CatalogInfo]
 
@@ -256,8 +241,8 @@ date_format_mappings = {
 date_format_re = re.compile('(%s)' % '|'.join(date_format_mappings))
 
 
-def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
-    # type: (datetime, str, str, Callable) -> str
+def babel_format_date(date: datetime, format: str, locale: str,
+                      formatter: Callable = babel.dates.format_date) -> str:
     if locale is None:
         locale = 'en'
 
@@ -277,8 +262,7 @@ def babel_format_date(date, format, locale, formatter=babel.dates.format_date):
         return format
 
 
-def format_date(format, date=None, language=None):
-    # type: (str, datetime, str) -> str
+def format_date(format: str, date: datetime = None, language: str = None) -> str:
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
         # See https://wiki.debian.org/ReproducibleBuilds/TimestampsProposal
@@ -286,7 +270,7 @@ def format_date(format, date=None, language=None):
         if source_date_epoch is not None:
             date = datetime.utcfromtimestamp(float(source_date_epoch))
         else:
-            date = datetime.now()
+            date = datetime.now(timezone.utc).astimezone()
 
     result = []
     tokens = date_format_re.split(format)
@@ -312,8 +296,7 @@ def format_date(format, date=None, language=None):
     return "".join(result)
 
 
-def get_image_filename_for_language(filename, env):
-    # type: (str, BuildEnvironment) -> str
+def get_image_filename_for_language(filename: str, env: "BuildEnvironment") -> str:
     if not env.config.language:
         return filename
 
@@ -332,8 +315,7 @@ def get_image_filename_for_language(filename, env):
         raise SphinxError('Invalid figure_language_filename: %r' % exc)
 
 
-def search_image_for_language(filename, env):
-    # type: (str, BuildEnvironment) -> str
+def search_image_for_language(filename: str, env: "BuildEnvironment") -> str:
     if not env.config.language:
         return filename
 
